@@ -89,7 +89,7 @@ struct _exsltDateValDate {
     unsigned int	min	:6;	/* 0 <=  min    <= 59	*/
     double		sec;
     unsigned int	tz_flag	:1;	/* is tzo explicitely set? */
-    int			tzo	:11;	/* -1440 <= tzo <= 1440 */
+    signed int		tzo	:12;	/* -1440 <= tzo <= 1440 currently only -840 to +840 are needed */
 };
 
 /* Duration value */
@@ -119,7 +119,8 @@ struct _exsltDateVal {
 
 #if defined(HAVE_TIME_H) 					\
     && (defined(HAVE_LOCALTIME) || defined(HAVE_LOCALTIME_R))	\
-    && defined(HAVE_TIME) && defined(HAVE_GMTIME)
+    && (defined(HAVE_GMTIME) || defined(HAVE_GMTIME_R))		\
+    && defined(HAVE_TIME)
 #define WITH_TIME
 #endif
 
@@ -551,12 +552,12 @@ _exsltDateParseTime (exsltDateValDatePtr dt, const xmlChar **str)
 static int
 _exsltDateParseTimeZone (exsltDateValDatePtr dt, const xmlChar **str)
 {
-    const xmlChar *cur = *str;
+    const xmlChar *cur;
     int ret = 0;
 
     if (str == NULL)
 	return -1;
-
+    cur = *str;
     switch (*cur) {
     case 0:
 	dt->tz_flag = 0;
@@ -744,9 +745,12 @@ static exsltDateValPtr
 exsltDateCurrent (void)
 {
     struct tm *localTm, *gmTm;
-    time_t secs;
+    time_t secs, gsecs;
 #if HAVE_LOCALTIME_R
     struct tm localTmS;
+#endif
+#if HAVE_GMTIME_R
+    struct tm gmTmS;
 #endif
     exsltDateValPtr ret;
 
@@ -775,13 +779,22 @@ exsltDateCurrent (void)
     ret->value.date.sec  = (double) localTm->tm_sec;
 
     /* determine the time zone offset from local to gm time */
+#if HAVE_GMTIME_R
+    gmtime_r(&secs, &gmTmS);
+    gmTm = &gmTmS;
+#else
     gmTm = gmtime(&secs);
+#endif
     ret->value.date.tz_flag = 0;
+#if 0
     ret->value.date.tzo = (((ret->value.date.day * 1440) +
                             (ret->value.date.hour * 60) +
                              ret->value.date.min) -
                            ((gmTm->tm_mday * 1440) + (gmTm->tm_hour * 60) +
                              gmTm->tm_min));
+#endif
+    gsecs = mktime(gmTm);
+    ret->value.date.tzo = (secs - gsecs) / 60;
 
     return ret;
 }
@@ -1508,10 +1521,18 @@ _exsltDateAdd (exsltDateValPtr dt, exsltDateValPtr dur)
 
     while (1) {
         if (tempdays < 1) {
-            long tmon = (long)MODULO_RANGE(r->mon-1, 1, 13);
-            long tyr  = r->year + (long)FQUOTIENT_RANGE(r->mon-1, 1, 13);
+            long tmon = (long)MODULO_RANGE((int)r->mon-1, 1, 13);
+            long tyr  = r->year + (long)FQUOTIENT_RANGE((int)r->mon-1, 1, 13);
             if (tyr == 0)
                 tyr--;
+	    /*
+	     * Coverity detected an overrun in daysInMonth 
+	     * of size 12 at position 12 with index variable "((r)->mon - 1)"
+	     */
+	    if (tmon < 0)
+	        tmon = 0;
+	    if (tmon > 12)
+	        tmon = 12;
             tempdays += MAX_DAYINMONTH(tyr, tmon);
             carry = -1;
         } else if (tempdays > (long)MAX_DAYINMONTH(r->year, r->mon)) {
@@ -1651,27 +1672,22 @@ _exsltDateDifference (exsltDateValPtr x, exsltDateValPtr y, int flag)
 }
 
 /**
- * _exsltDateAddDuration:
- * @x: an #exsltDateValPtr of type #XS_DURATION
- * @y: an #exsltDateValPtr of type #XS_DURATION
+ * _exsltDateAddDurCalc
+ * @ret: an exsltDateValPtr for the return value:
+ * @x: an exsltDateValPtr for the first operand
+ * @y: an exsltDateValPtr for the second operand
  *
- * Compute a new duration from @x and @y.
+ * Add two durations, catering for possible negative values.
+ * The sum is placed in @ret.
  *
- * Returns date/time pointer or NULL.
+ * Returns 1 for success, 0 if error detected.
  */
-static exsltDateValPtr
-_exsltDateAddDuration (exsltDateValPtr x, exsltDateValPtr y)
+static int
+_exsltDateAddDurCalc (exsltDateValPtr ret, exsltDateValPtr x,
+		      exsltDateValPtr y)
 {
-    exsltDateValPtr ret;
     long carry;
 
-    if ((x == NULL) || (y == NULL))
-        return NULL;
-
-    ret = exsltDateCreateDate(XS_DURATION);
-    if (ret == NULL)
-        return NULL;
-
     /* months */
     ret->value.dur.mon = x->value.dur.mon + y->value.dur.mon;
 
@@ -1680,6 +1696,16 @@ _exsltDateAddDuration (exsltDateValPtr x, exsltDateValPtr y)
     carry = (long)FQUOTIENT(ret->value.dur.sec, SECS_PER_DAY);
     if (ret->value.dur.sec != 0.0) {
         ret->value.dur.sec = MODULO(ret->value.dur.sec, SECS_PER_DAY);
+	/*
+	 * Our function MODULO always gives us a positive value, so
+	 * if we end up with a "-ve" carry we need to adjust it
+	 * appropriately (bug 154021)
+	 */
+	if ((carry < 0) && (ret->value.dur.sec != 0)) {
+	    /* change seconds to equiv negative modulus */
+	    ret->value.dur.sec = ret->value.dur.sec - SECS_PER_DAY;
+	    carry++;
+	}
     }
 
     /* days */
@@ -1693,11 +1719,37 @@ _exsltDateAddDuration (exsltDateValPtr x, exsltDateValPtr y)
          (ret->value.dur.mon < 0)) ||
         (((ret->value.dur.day < 0) || (ret->value.dur.sec < 0)) &&
          (ret->value.dur.mon > 0))) {
-	exsltDateFreeDate(ret);
-        return NULL;
+        return 0;
     }
+    return 1;
+}
 
-    return ret;
+/**
+ * _exsltDateAddDuration:
+ * @x: an #exsltDateValPtr of type #XS_DURATION
+ * @y: an #exsltDateValPtr of type #XS_DURATION
+ *
+ * Compute a new duration from @x and @y.
+ *
+ * Returns date/time pointer or NULL.
+ */
+static exsltDateValPtr
+_exsltDateAddDuration (exsltDateValPtr x, exsltDateValPtr y)
+{
+    exsltDateValPtr ret;
+
+    if ((x == NULL) || (y == NULL))
+        return NULL;
+
+    ret = exsltDateCreateDate(XS_DURATION);
+    if (ret == NULL)
+        return NULL;
+
+    if (_exsltDateAddDurCalc(ret, x, y))
+        return ret;
+
+    exsltDateFreeDate(ret);
+    return NULL;
 }
 
 /****************************************************************
@@ -2101,8 +2153,8 @@ exsltDateWeekInYear (const xmlChar *dateTime)
      */
     fdiw = (_exsltDateDayInWeek(fdiy, dt->value.date.year) + 6) % 7;
 
-    ret = DAY_IN_YEAR(dt->value.date.day, dt->value.date.mon,
-                      dt->value.date.year) / 7;
+    ret = (DAY_IN_YEAR(dt->value.date.day, dt->value.date.mon,
+                      dt->value.date.year) + fdiw) / 7;
 
     /* ISO 8601 adjustment, 3 is Thu */
     if (fdiw <= 3)
@@ -2163,7 +2215,7 @@ exsltDateWeekInMonth (const xmlChar *dateTime)
      */
     fdiw = (_exsltDateDayInWeek(fdiy, dt->value.date.year) + 6) % 7;
 
-    ret = ((dt->value.date.day + fdiw) / 7) + 1;
+    ret = ((dt->value.date.day + fdiw - 1) / 7) + 1;
 
     exsltDateFreeDate(dt);
 
@@ -2308,7 +2360,7 @@ exsltDateDayOfWeekInMonth (const xmlChar *dateTime)
 	}
     }
 
-    ret = (dt->value.date.day / 7) + 1;
+    ret = ((dt->value.date.day -1) / 7) + 1;
 
     exsltDateFreeDate(dt);
 
@@ -2729,7 +2781,6 @@ exsltDateSumFunction (xmlXPathParserContextPtr ctxt, int nargs)
     xmlChar *tmp;
     exsltDateValPtr x, total;
     xmlChar *ret;
-    long carry;
     int i;
 
     if (nargs != 1) {
@@ -2762,7 +2813,7 @@ exsltDateSumFunction (xmlXPathParserContextPtr ctxt, int nargs)
     }
 
     for (i = 0; i < ns->nodeNr; i++) {
-
+    	int result;
 	tmp = xmlXPathCastNodeToString (ns->nodeTab[i]);
 	if (tmp == NULL) {
 	    xmlXPathFreeNodeSet (ns);
@@ -2779,22 +2830,16 @@ exsltDateSumFunction (xmlXPathParserContextPtr ctxt, int nargs)
 	    return;
 	}
 
-	/* months */
-	total->value.dur.mon += x->value.dur.mon;
-
-	/* seconds */
-	total->value.dur.sec += x->value.dur.sec;
-	carry = (long) FQUOTIENT (total->value.dur.sec, SECS_PER_DAY);
-	if (total->value.dur.sec != 0.0) {
-	    total->value.dur.sec =
-		MODULO (total->value.dur.sec, SECS_PER_DAY);
-	}
-
-	/* days */
-	total->value.dur.day += x->value.dur.day + carry;
+	result = _exsltDateAddDurCalc(total, total, x);
 
 	exsltDateFreeDate (x);
 	xmlFree (tmp);
+	if (!result) {
+	    exsltDateFreeDate (total);
+	    xmlXPathFreeNodeSet (ns);
+	    xmlXPathReturnEmptyString (ctxt);
+	    return;
+	}
     }
 
     ret = exsltDateFormatDuration (&(total->value.dur));
