@@ -324,6 +324,7 @@ xsltExtModuleRegisterDynamic(const xmlChar * URI)
     const xmlChar *ext_directory = NULL;
     const xmlChar *protocol = NULL;
     xmlChar *i, *regfunc_name;
+    void *vregfunc;
     int rc;
 
     /* check for bad inputs */
@@ -414,9 +415,15 @@ xsltExtModuleRegisterDynamic(const xmlChar * URI)
     regfunc_name = xmlStrdup(ext_name);
     regfunc_name = xmlStrcat(regfunc_name, BAD_CAST "_init");
 
-    rc = xmlModuleSymbol(m, (const char *) regfunc_name, (void **) &regfunc);
+    vregfunc = NULL;
+    rc = xmlModuleSymbol(m, (const char *) regfunc_name, &vregfunc);
+    regfunc = vregfunc;
     if (0 == rc) {
-        /* call the module's init function */
+        /*
+	 * Call the module's init function.  Note that this function
+	 * calls xsltRegisterExtModuleFull which will add the module
+	 * to xsltExtensionsHash (together with it's entry points).
+	 */
         (*regfunc) ();
 
         /* register this module in our hash */
@@ -519,12 +526,12 @@ xsltRegisterExtPrefix(xsltStylesheetPtr style,
     style->nsDefs = ret;
 
     /*
-     * check wether there is an extension module with a stylesheet
+     * check whether there is an extension module with a stylesheet
      * initialization function.
      */
 #ifdef XSLT_REFACTORED
     /*
-    * Don't initialize modules based on specified namespaced via
+    * Don't initialize modules based on specified namespaces via
     * the attribute "[xsl:]extension-element-prefixes".
     */
 #else
